@@ -19,6 +19,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include <math.h>
 #include <limits.h>
 #include <assert.h>
 #include "config.h"
@@ -31,8 +32,8 @@
 /* A default palette, containing standard ANSI / EGA colors
  *
  */
-static unsigned char defpal_data[4*16] = 
-{  
+static unsigned char defpal_data[4*16] =
+{
 0  ,0  ,0  ,255,
 127,0  ,0  ,255,
 0  ,127,0  ,255,
@@ -52,6 +53,14 @@ static unsigned char defpal_data[4*16] =
 };
 static double defpal_double[4*8*16];
 
+static unsigned short ceil_sqrt_u8[3 * 255 * 255 + 1];
+
+
+typedef struct BablPaletteRadius
+{
+  unsigned char  idx;
+  unsigned short diff;
+} BablPaletteRadius;
 
 typedef struct BablPalette
 {
@@ -62,9 +71,87 @@ typedef struct BablPalette
                                   */
   double                *data_double;
   unsigned char         *data_u8;
+  BablPaletteRadius     *radii;
   volatile unsigned int  hash[HASH_TABLE_SIZE];
 } BablPalette;
 
+static void
+init_ceil_sqrt_u8 (void)
+{
+  int i;
+
+  babl_mutex_lock (babl_format_mutex);
+
+  if (! ceil_sqrt_u8[1])
+    {
+      for (i = 0; i <= 3 * 255 * 255; i++)
+        ceil_sqrt_u8[i] = ceil (sqrt (i));
+    }
+
+  babl_mutex_unlock (babl_format_mutex);
+}
+
+static inline int
+diff2_u8 (const unsigned char *p1,
+          const unsigned char *p2)
+{
+  return ((int) p1[0] - (int) p2[0]) * ((int) p1[0] - (int) p2[0]) +
+         ((int) p1[1] - (int) p2[1]) * ((int) p1[1] - (int) p2[1]) +
+         ((int) p1[2] - (int) p2[2]) * ((int) p1[2] - (int) p2[2]);
+}
+
+static int
+babl_palette_radius_compare (const void *r1,
+                             const void *r2)
+{
+  const BablPaletteRadius *radius1 = r1;
+  const BablPaletteRadius *radius2 = r2;
+
+  return (int) radius1->diff - (int) radius2->diff;
+}
+
+static void
+babl_palette_init_radii (BablPalette *pal)
+{
+  int i, j;
+
+  init_ceil_sqrt_u8 ();
+
+  /* calculate the distance between each pair of colors in the palette, and, for
+   * each color, construct a list of all other colors and their distances from
+   * it, sorted by distance.  we use these lists in babl_palette_lookup() to
+   * speed up the search, as described in the function.
+   */
+
+  pal->radii = babl_malloc (sizeof (BablPaletteRadius) *
+                            (pal->count - 1)           *
+                            pal->count);
+
+  for (i = 0; i < pal->count; i++)
+    {
+      BablPaletteRadius   *radii1 = pal->radii + (pal->count - 1) * i;
+      const unsigned char *p1     = pal->data_u8 + 4 * i;
+
+      for (j = i + 1; j < pal->count; j++)
+        {
+          BablPaletteRadius   *radii2 = pal->radii + (pal->count - 1) * j;
+          const unsigned char *p2     = pal->data_u8 + 4 * j;
+          unsigned short       diff;
+
+          diff = floor (sqrt (diff2_u8 (p1, p2)));
+
+          radii1[j - 1].idx  = j;
+          radii1[j - 1].diff = diff;
+
+          radii2[i].idx      = i;
+          radii2[i].diff     = diff;
+        }
+
+      qsort (radii1, pal->count - 1, sizeof (BablPaletteRadius),
+             babl_palette_radius_compare);
+    }
+}
+
 static void
 babl_palette_reset_hash (BablPalette *pal)
 {
@@ -75,10 +162,14 @@ babl_palette_reset_hash (BablPalette *pal)
     }
 }
 
+#define BABL_IDX_FACTOR 255.5
+
 static int
-babl_palette_lookup (BablPalette *pal, int r, int g, int b, int a)
+babl_palette_lookup (BablPalette         *pal,
+                     const unsigned char *p,
+                     int                  best_idx)
 {
-  unsigned int pixel      = (r << 16) | (g << 8) | b;
+  unsigned int pixel      = p[0] | (p[1] << 8) | (p[2] << 16);
   int          hash_index = pixel % HASH_TABLE_SIZE;
   unsigned int hash_value = pal->hash[hash_index];
   unsigned int hash_pixel = hash_value & 0x00ffffffu;
@@ -97,26 +188,60 @@ babl_palette_lookup (BablPalette *pal, int r, int g, int b, int a)
     }
   else
     {
-      int best_idx = 0;
-      int best_diff = INT_MAX;
+      const BablPaletteRadius *radii = pal->radii + (pal->count - 1) * best_idx;
+      const unsigned char     *q;
+      int                      best_diff2;
+      int                      best_diff;
+      int                      diff0;
+      int                      i;
 
-      for (idx = 0; idx < pal->count; idx++)
+      /* best_idx is the closest palette entry to the previous pixel (referred
+       * to as the source color).  based on the assumption that nearby pixels
+       * have similar color, we start the search for the current closest entry
+       * at best_idx, and iterate over the entry's color list, as calculated in
+       * babl_palette_init_radii(), in search for a better match.
+       */
+
+      q          = pal->data_u8 + 4 * best_idx;
+      best_diff2 = diff2_u8 (p, q);
+      best_diff  = ceil_sqrt_u8[best_diff2];
+      diff0      = best_diff;
+
+      for (i = 0; i < pal->count - 1; i++)
         {
-          unsigned char *palpx = pal->data_u8 + idx * 4;
-          int pr = palpx[0];
-          int pg = palpx[1];
-          int pb = palpx[2];
+          const BablPaletteRadius *radius = &radii[i];
+          int                      min_diff;
+          int                      diff2;
 
-          int diff = (r - pr) * (r - pr) +
-                     (g - pg) * (g - pg) +
-                     (b - pb) * (b - pb);
-          if (diff < best_diff)
+          /* radius->diff is the distance from the source color to the current
+           * color.  diff0 is the distance from the source color to the input
+           * color.  according to the triangle inequality, the distance from
+           * the current color to the input color is at least
+           * radius->diff - diff0.  if the shortest distance found so far is
+           * less than that, then the best match found so far is necessarily
+           * better than the current color, and we can stop the search, since
+           * the color list is sorted in ascending radius->diff order.
+           */
+
+          idx      = radius->idx;
+          min_diff = radius->diff - diff0;
+
+          if (best_diff < min_diff || (best_diff == min_diff && best_idx < idx))
+            break;
+
+          q     = pal->data_u8 + 4 * idx;
+          diff2 = diff2_u8 (p, q);
+
+          if (diff2 < best_diff2 || (diff2 == best_diff2 && idx < best_idx))
             {
-              best_diff = diff;
-              best_idx  = idx;
+              best_idx   = idx;
+              best_diff2 = diff2;
+              best_diff  = ceil_sqrt_u8[diff2];
             }
         }
+
       pal->hash[hash_index] = ((unsigned int) best_idx << 24) | pixel;
+
       return best_idx;
     }
 }
@@ -139,6 +264,8 @@ static BablPalette *make_pal (const Babl *format, const void *data, int count)
   babl_process (babl_fish (format, babl_format ("R'G'B'A u8")),
                 data, pal->data_u8, count);
 
+  babl_palette_init_radii (pal);
+
   babl_palette_reset_hash (pal);
 
   return pal;
@@ -149,6 +276,7 @@ static void babl_palette_free (BablPalette *pal)
   babl_free (pal->data);
   babl_free (pal->data_double);
   babl_free (pal->data_u8);
+  babl_free (pal->radii);
   babl_free (pal);
 }
 
@@ -172,93 +300,101 @@ static BablPalette *default_palette (void)
   babl_process (babl_fish (pal.format, babl_format ("RGBA double")),
                 pal.data, pal.data_double, pal.count);
 
+  babl_palette_init_radii (&pal);
   babl_palette_reset_hash (&pal);
   return &pal;
 }
 
 static void
 rgba_to_pal (Babl *conversion,
-             char *src,
+             char *src_b,
              char *dst,
              long  n,
              void *dst_model_data)
 {
+  const Babl *space = babl_conversion_get_source_space (conversion);
   BablPalette **palptr = dst_model_data;
-  BablPalette *pal = *palptr;
+  BablPalette *pal;
+  int best_idx = 0;
+  assert (palptr);
+  pal = *palptr;
+  assert(pal);
+
   while (n--)
     {
-      int idx;
+      double *src_d = (void*) src_b;
+      unsigned char src[4];
+      int c;
+      for (c = 0; c < 3; c++)
+      {
+        if (src_d[c] >= 1.0f)
+          src[c] = 255;
+        else if (src_d[c] <= 0.0f)
+          src[c] = 0;
+        else
+          src[c] = babl_trc_from_linear (space->space.trc[0],
+                                         src_d[c]) * 255 + 0.5f;
+      }
+      if (src_d[3] >= 1.0f)
+        src[3] = 255;
+      else if (src_d[3] <= 0.0f)
+        src[3] = 0;
+      else
+        src[3] = src_d[3] * 255 + 0.5f;
 
-      int best_idx = 0;
-      double best_diff = 100000;
-      double *srcf;
+      best_idx = babl_palette_lookup (pal, src, best_idx);
 
-      srcf = ((double *) src);
+      ((double *) dst)[0] = best_idx / BABL_IDX_FACTOR;
 
-      for (idx = 0; idx<pal->count; idx++)
-        {
-          double diff;
-          double *palpx = ((double *)pal->data_double) + idx * 4;
-
-          diff = (palpx[0] - srcf[0]) * (palpx[0] - srcf[0]) +
-                 (palpx[1] - srcf[1]) * (palpx[1] - srcf[1]) +
-                 (palpx[2] - srcf[2]) * (palpx[2] - srcf[2]);
-          if (diff <= best_diff)
-            {
-              best_diff = diff;
-              best_idx = idx;
-            }
-        }
-
-      ((double *) dst)[0] = best_idx / 255.5;
-
-      src += sizeof (double) * 4;
+      src_b += sizeof (double) * 4;
       dst += sizeof (double) * 1;
     }
+
 }
 
 static void
 rgba_to_pala (Babl *conversion,
-              char *src,
+              char *src_i,
               char *dst,
               long  n,
               void *dst_model_data)
 {
+  const Babl *space = babl_conversion_get_destination_space (conversion);
   BablPalette **palptr = dst_model_data;
-  BablPalette *pal = *palptr;
-  
+  BablPalette *pal;
+  int best_idx = 0;
+  assert (palptr);
+  pal = *palptr;
   assert(pal);
+
   while (n--)
     {
-      int idx;
+      double *src_d = (void*) src_i;
+      unsigned char src[4];
+      int c;
+      for (c = 0; c < 3; c++)
+      {
+        if (src_d[c] >= 1.0f)
+          src[c] = 255;
+        else if (src_d[c] <= 0.0f)
+          src[c] = 0;
+        else
+          src[c] = babl_trc_from_linear (space->space.trc[0],
+                                         src_d[c]) * 255 + 0.5f;
+      }
+      if (src_d[3] >= 1.0f)
+        src[3] = 255;
+      else if (src_d[3] <= 0.0f)
+        src[3] = 0;
+      else
+        src[3] = src_d[3] * 255 + 0.5f;
 
-      int best_idx = 0;
-      double best_diff = 100000;
-      double *srcf;
-      double alpha;
+      best_idx = babl_palette_lookup (pal, src, best_idx);
 
-      srcf = ((double *) src);
-      alpha = srcf[3];
+      ((double *) dst)[0] = best_idx / BABL_IDX_FACTOR;
+      ((double *) dst)[1] = src_d[3];
 
-      for (idx = 0; idx<pal->count; idx++)
-        {
-          double diff;
-          double *palpx = ((double *)pal->data_double) + idx * 4;
-
-          diff = (palpx[0] - srcf[0]) * (palpx[0] - srcf[0]) +
-                 (palpx[1] - srcf[1]) * (palpx[1] - srcf[1]) +
-                 (palpx[2] - srcf[2]) * (palpx[2] - srcf[2]);
-          if (diff <= best_diff)
-            {
-              best_diff = diff;
-              best_idx = idx;
-            }
-        }
-
-      ((double *) dst)[0] = best_idx / 255.5;
-      ((double *) dst)[1] = alpha;
-
-      src += sizeof (double) * 4;
+      src_i += sizeof (double) * 4;
       dst += sizeof (double) * 2;
     }
 }
@@ -275,7 +411,7 @@ pal_to_rgba (Babl *conversion,
   assert(pal);
   while (n--)
     {
-      int idx = (((double *) src)[0]) * 255.5;
+      int idx = (((double *) src)[0]) * BABL_IDX_FACTOR;
       double *palpx;
 
       if (idx < 0) idx = 0;
@@ -302,7 +438,7 @@ pala_to_rgba (Babl *conversion,
   assert(pal);
   while (n--)
     {
-      int idx      = (((double *) src)[0]) * 255.5;
+      int idx      = (((double *) src)[0]) * BABL_IDX_FACTOR;
       double alpha = (((double *) src)[1]);
       double *palpx;
 
@@ -312,13 +448,104 @@ pala_to_rgba (Babl *conversion,
       palpx = ((double *)pal->data_double) + idx * 4;
       memcpy (dst, palpx, sizeof(double)*4);
 
-      ((double *)dst)[3] *= alpha; 
+      ((double *)dst)[3] *= alpha;
 
       src += sizeof (double) * 2;
       dst += sizeof (double) * 4;
     }
 }
 
+static void
+rgba_float_to_pal_a (Babl          *conversion,
+                     unsigned char *src_b,
+                     unsigned char *dst,
+                     long           n,
+                     void          *src_model_data)
+{
+  const Babl *space = babl_conversion_get_destination_space (conversion);
+  BablPalette **palptr = src_model_data;
+  BablPalette *pal;
+  int best_idx = 0;
+  assert (palptr);
+  pal = *palptr;
+  assert(pal);
+
+  while (n--)
+    {
+      float *src_f = (void*) src_b;
+      unsigned char src[4];
+      int c;
+      for (c = 0; c < 3; c++)
+      {
+        if (src_f[c] >= 1.0f)
+          src[c] = 255;
+        else if (src_f[c] <= 0.0f)
+          src[c] = 0;
+        else
+          src[c] = babl_trc_from_linear (space->space.trc[0],
+                                         src_f[c]) * 255 + 0.5f;
+      }
+      if (src_f[3] >= 1.0f)
+        src[3] = 255;
+      else if (src_f[3] <= 0.0f)
+        src[3] = 0;
+      else
+        src[3] = src_f[3] * 255 + 0.5f;
+
+
+      dst[0] = best_idx = babl_palette_lookup (pal, src, best_idx);
+      dst[1] = src[3];
+
+      src_b += sizeof (float) * 4;
+      dst += sizeof (char) * 2;
+    }
+}
+
+
+static void
+rgba_float_to_pal (Babl          *conversion,
+                   unsigned char *src_b,
+                   unsigned char *dst,
+                   long           n,
+                   void          *src_model_data)
+{
+  const Babl *space = babl_conversion_get_destination_space (conversion);
+  BablPalette **palptr = src_model_data;
+  BablPalette *pal;
+  int best_idx = 0;
+  assert (palptr);
+  pal = *palptr;
+  assert(pal);
+
+  while (n--)
+    {
+      float *src_f = (void*) src_b;
+      unsigned char src[4];
+      int c;
+      for (c = 0; c < 3; c++)
+      {
+        if (src_f[c] >= 1.0f)
+          src[c] = 255;
+        else if (src_f[c] <= 0.0f)
+          src[c] = 0;
+        else
+          src[c] = babl_trc_from_linear (space->space.trc[0],
+                                         src_f[c]) * 255 + 0.5f;
+      }
+      if (src_f[3] >= 1.0f)
+        src[3] = 255;
+      else if (src_f[3] <= 0.0f)
+        src[3] = 0;
+      else
+        src[3] = src_f[3] * 255 + 0.5f;
+
+      dst[0] = best_idx = babl_palette_lookup (pal, src, best_idx);
+
+      src_b += sizeof (float) * 4;
+      dst += sizeof (char) * 1;
+    }
+}
+
 static void
 rgba_u8_to_pal (Babl          *conversion,
                 unsigned char *src,
@@ -328,12 +555,14 @@ rgba_u8_to_pal (Babl          *conversion,
 {
   BablPalette **palptr = src_model_data;
   BablPalette *pal;
+  int best_idx = 0;
   assert (palptr);
   pal = *palptr;
   assert(pal);
+
   while (n--)
     {
-      dst[0] = babl_palette_lookup (pal, src[0], src[1], src[2], src[3]);
+      dst[0] = best_idx = babl_palette_lookup (pal, src, best_idx);
 
       src += sizeof (char) * 4;
       dst += sizeof (char) * 1;
@@ -349,12 +578,13 @@ rgba_u8_to_pal_a (Babl          *conversion,
 {
   BablPalette **palptr = src_model_data;
   BablPalette *pal;
+  int best_idx = 0;
   assert (palptr);
   pal = *palptr;
   assert(pal);
   while (n--)
     {
-      dst[0] = babl_palette_lookup (pal, src[0], src[1], src[2], src[3]);
+      dst[0] = best_idx = babl_palette_lookup (pal, src, best_idx);
       dst[1] = src[3];
 
       src += sizeof (char) * 4;
@@ -510,7 +740,6 @@ const Babl *babl_new_palette (const char  *name,
     "chroma",
     NULL);
   alpha = babl_component ("A");
-  
   model = babl_model_new ("name", name, component, alpha, NULL);
   palptr = malloc (sizeof (void*));
   *palptr = default_palette ();;
@@ -558,30 +787,24 @@ const Babl *babl_new_palette (const char  *name,
      "data", palptr,
      NULL
   );
-
   babl_conversion_new (
      f_pal_u8,
      f_pal_a_u8,
      "linear", conv_pal8_pala8,
      NULL
   );
-
   babl_conversion_new (
      f_pal_a_u8,
      f_pal_u8,
      "linear", conv_pala8_pal8,
      NULL
   );
-
-
   babl_conversion_new (
      f_pal_u8,
      babl_format ("R'G'B'A u8"),
      "linear", pal_u8_to_rgba_u8,
      "data", palptr,
      NULL);
-
-
   babl_conversion_new (
      f_pal_a_u8,
      babl_format ("R'G'B'A u8"),
@@ -602,6 +825,19 @@ const Babl *babl_new_palette (const char  *name,
      "data", palptr,
      NULL);
 
+  babl_conversion_new (
+     babl_format ("RGBA float"),
+     f_pal_a_u8,
+     "linear", rgba_float_to_pal_a,
+     "data", palptr,
+     NULL);
+  babl_conversion_new (
+     babl_format ("RGBA float"),
+     f_pal_u8,
+     "linear", rgba_float_to_pal,
+     "data", palptr,
+     NULL);
+
   babl_set_user_data (model, palptr);
   babl_set_user_data (model_no_alpha, palptr);
 
